@@ -5,7 +5,7 @@
 *                                                                           *
 *  Copyright 2009-2010 Geert Mulders <g.c.w.m.mulders@gmail.com>            *
 *  Copyright 2010-2013 Wilmer van der Gaast <wilmer@gaast.net>              *
-*  Copyright 2017-2018 Alex Schroeder <alex@gnu.org>                        *
+*  Copyright 2017-2019 Alex Schroeder <alex@gnu.org>                        *
 *                                                                           *
 *  This library is free software; you can redistribute it and/or            *
 *  modify it under the terms of the GNU Lesser General Public               *
@@ -337,9 +337,6 @@ static void mastodon_connect(struct im_connection *ic)
 	md->url_ssl = url.proto == PROTO_HTTPS; // always
 	md->url_port = url.port;
 	md->url_host = g_strdup(url.host);
-	if (strcmp(url.file, "/") != 0) {
-		md->url_path = g_strdup(url.file);
-	}
 
 	mastodon_set_name(ic);
 	imcb_add_buddy(ic, md->name, NULL);
@@ -428,11 +425,6 @@ static void mastodon_login(account_t * acc)
 	md->url_ssl = 1;
 	md->url_port = url.port;
 	md->url_host = g_strdup(url.host);
-	if (strcmp(url.file, "/") != 0) {
-		md->url_path = g_strdup(url.file);
-	} else {
-		md->url_path = g_strdup("");
-	}
  	mastodon_set_name(ic);
 
 	GSList *p_in = NULL;
@@ -515,7 +507,6 @@ static void mastodon_logout(struct im_connection *ic)
 		g_free(md->name); md->name = NULL;
 		g_free(md->next_url); md->next_url = NULL;
 		g_free(md->url_host); md->url_host = NULL;
-		g_free(md->url_path); md->url_path = NULL;
 		g_free(md);
 		ic->proto_data = NULL;
 	}
@@ -734,14 +725,14 @@ finish:
 static void mastodon_handle_command(struct im_connection *ic, char *message, mastodon_undo_t undo_type);
 
 /**
- * Send a direct message. If this buddy is the magic mastodon oauth
- * handle, then treat the message as the refresh token. If this buddy
- * is me, then treat the message as a command.
+ * Send a direct message. If this buddy is the magic mastodon oauth handle, then treat the message as the refresh token.
+ * If this buddy is me, then treat the message as a command. Everything else is a message to a buddy in a query.
  */
 static int mastodon_buddy_msg(struct im_connection *ic, char *who, char *message, int away)
 {
 	struct mastodon_data *md = ic->proto_data;
 
+	/* OAuth message to "mastodon_oauth" */
 	if (g_ascii_strcasecmp(who, MASTODON_OAUTH_HANDLE) == 0 &&
 	    !(md->flags & OPT_LOGGED_IN)) {
 
@@ -755,9 +746,20 @@ static int mastodon_buddy_msg(struct im_connection *ic, char *who, char *message
 	}
 
 	if (g_ascii_strcasecmp(who, md->name) == 0) {
+		/* Message to ourselves */
 		mastodon_handle_command(ic, message, MASTODON_NEW);
 	} else {
-		mastodon_post_message(ic, message, 0, who, MASTODON_REPLY, NULL, MV_DIRECT, NULL);
+		/* Determine who and to what post id we are replying to */
+		guint64 in_reply_to = 0;
+		bee_user_t *bu;
+		if ((bu = bee_user_by_handle(ic->bee, ic, who))) {
+			struct mastodon_user_data *mud = bu->data;
+			if (time(NULL) < mud->last_direct_time + set_getint(&ic->acc->set, "auto_reply_timeout")) {
+				/* this is a reply */
+				in_reply_to = mud->last_direct_id;
+			}
+		}
+		mastodon_post_message(ic, message, in_reply_to, who, MASTODON_REPLY, NULL, MV_DIRECT, NULL);
 	}
 	return 0;
 }
